from __future__ import unicode_literals
from abc import ABCMeta, abstractmethod
from six import with_metaclass
from pygments.token import Token

__all__ = (
    'PromptStyle',
    'IPythonPrompt',
    'ClassicPrompt',
    'PseudoTerminalPrompt'
)


class PromptStyle(with_metaclass(ABCMeta, object)):
    """
    Base class for all prompts.
    """
    @abstractmethod
    def in_tokens(self, cli):
        " Return the input tokens. "
        return []

    @abstractmethod
    def in2_tokens(self, cli, width):
        """
        Tokens for every following input line.

        :param width: The available width. This is coming from the width taken
                      by `in_tokens`.
        """
        return []

    @abstractmethod
    def out_tokens(self, cli):
        " Return the output tokens. "
        return []


class IPythonPrompt(PromptStyle):
    """
    A prompt resembling the IPython prompt.
    """
    def __init__(self, python_input):
        self.python_input = python_input

    def in_tokens(self, cli):
        return [
            (Token.In, 'In ['),
            (Token.In.Number, '%s' % self.python_input.current_statement_index),
            (Token.In, ']: '),
        ]

    def in2_tokens(self, cli, width):
        return [
            (Token.In, '...: '.rjust(width)),
        ]

    def out_tokens(self, cli):
        return [
            (Token.Out, 'Out['),
            (Token.Out.Number, '%s' % self.python_input.current_statement_index),
            (Token.Out, ']:'),
            (Token, ' '),
        ]


class ClassicPrompt(PromptStyle):
    """
    The classic Python prompt.
    """
    def in_tokens(self, cli):
        return [(Token.Prompt, '>>> ')]

    def in2_tokens(self, cli, width):
        return [(Token.Prompt.Dots, '...')]

    def out_tokens(self, cli):
        return []
class PseudoTerminalPrompt(ClassicPrompt):
    """
    The classic Python prompt.
    """
    def in_tokens(self, cli):

        return [(Token.Comment, ' ⮤ ')]




import time
import random
import os
dir_path = os.path.dirname(os.path.realpath(__file__))
import sys
sys.path.append(os.path.join(dir_path,'graphics'))
sys.path.append(dir_path)

import letters
import graphics
from graphics.nbinput import NonBlockingInput
import tetrisShapes as tetShapes

def contains(arr, n):
    try:
        arr.index(n)
        return True
    except ValueError:
        return False

class Tetris(graphics.Canvas):
    def __init__(self, size=(20, 30)):
        super().__init__(size)

    @property
    def alive(self):
        highShapes = 0
        for shape in self.sprites:
            if shape.position[1] <= 10:
                highShapes += 1
        if highShapes > 1:
            return False
        else:
            return True

class Shape(graphics.Sprite):
    def __init__(self, image, position=(0, 0)):
        super().__init__(image, position, color=image.n)

def main():
    game = Tetris()

    shape = Shape(tetShapes.Shape(random.randint(0, 6)), (0, 10))
    game.sprites.append(shape)

    shapeNo = 0
    lastFrame = time.time()
    lastDownMv = time.time()
    lastBtn = time.time()
    start = time.time()
    with NonBlockingInput() as nbi:
        while game.alive:

            if time.time() > lastFrame + 0.1:
                print(game)
                lastFrame = time.time()

            if contains(shape.onEdge(game), graphics.BOTTOM) or shape.touching(game, side=graphics.BOTTOM):
                shape = Shape(tetShapes.Shape(random.randint(0, 6)), (0, 10))
                game.sprites.append(shape)
                shapeNo += 1
            else:

                if time.time() > lastDownMv + 0.5:
                    shape.move(graphics.DOWN)
                    lastDownMv = time.time()

                ch = nbi.char()
                # Rotate?
                if ch == ' ' and time.time() > lastBtn + 0.05:
                    lastBtn = time.time()

                    shape.image.rotate(graphics.LEFT)
                    if contains(shape.onEdge(game), graphics.BOTTOM) or shape.overlaps(game):
                        shape.image.rotate(graphics.RIGHT)

                # Move?
                if ch == ',' or ch == '/' or ch == '.' and time.time() > lastBtn + 0.05:
                    lastBtn = time.time()

                    if ch == ',':
                        t = graphics.LEFT
                    elif ch == '/':
                        t = graphics.RIGHT
                    elif ch == '.':
                        t = graphics.DOWN

                    shape.move(t)

                    if (contains(shape.onEdge(game), graphics.LEFT) or
                        contains(shape.onEdge(game), graphics.BOTTOM) or
                        contains(shape.onEdge(game), graphics.RIGHT) or
                        shape.overlaps(game)):

                        if t == graphics.LEFT:  shape.move(graphics.RIGHT)
                        if t == graphics.DOWN:  shape.move(graphics.UP)
                        if t == graphics.RIGHT: shape.move(graphics.LEFT)

    totalTime = time.time() - start
    time.sleep(1)

    size = graphics.console.Size().getSize()

    y = lambda: int((size[1]-5)/2)
    x = lambda l: int((size[0]-l)/2)

    text = letters.word('Game Over!')
    line = ''
    lineNo = 0
    print(y()*'\n')
    for letter in text:
        if letter == '\n':
            print(x(len(line))*' '+ line)
            line = ''
            lineNo += 1
        else:
            line += letter
    print(y()*'\n')

    print('You scored {score}!'.format( score = int( (shapeNo / totalTime) *100 )) )
    time.sleep(1)

if True or __name__ == '__main__':
    main()
    assert False,'Game over. Use "import tetris" again to play again'



"""
Styling for prompt_toolkit applications.
"""
from __future__ import unicode_literals

from .base import *
from .defaults import *
from .from_dict import *
from .from_pygments import *
from .utils import *


#: The default built-in style.
#: (For backwards compatibility, when Pygments is installed, this includes the
#: default Pygments style.)
try:
    import pygments
except ImportError:
    DEFAULT_STYLE = style_from_dict(DEFAULT_STYLE_EXTENSIONS)
else:
    DEFAULT_STYLE = style_from_pygments()



from __future__ import unicode_literals

commands_with_help = {
    'alias': "Creates an alias called 'name' the executes 'command'.",
    'args': 'Print the arguments of the current function.',
    'break': 'Set a break.',
    'clear': 'Clear breakpoints.',
    'commands': 'Specify a list of commands for breakpoint number bpnumber.',
    'condition': 'Make breakpoint conditional.',
    'cont': 'Continue execution, only stop when a breakpoint is encountered.',
    'continue': 'Continue execution, only stop when a breakpoint is encountered.',
    'debug': 'Enter a recursive debugger.',
    'disable': 'Disable breakpoints.',
    'down': 'Move the current frame one level down in the stack trace.',
    'enable': 'Enables breakpoints.',
    'exit': 'The program being executed is aborted.',
    'help': 'Help.',
    'ignore': 'Sets the ignore count for the given breakpoint number.',
    'jump': 'Set the next line that will be executed.',
    'list': 'List source code.',
    'next': 'Continue execution until the next line.',
    'p': 'Print the value of the expression.',
    'pp': 'Pretty-print expression.',
    'quit': 'The program being executed is aborted.',
    'restart': 'Restart the debugged python program.',
    'return': 'Continue execution until the current function returns.',
    'run': 'Restart the debugged python program.',
    'step': 'Execute the current line, stop at the first possible occasion.',
    'tbreak': 'Set a break, remove when first hit.',
    'unalias': 'Delete an alias.',
    'up': 'Move the current frame one level up in the stack trace.',
    'whatis': 'Prints the type of the argument.',
    'where': 'Print a stack trace.',
    'until': 'Continue execution until.',
    'display': 'Display the value of the expression if it changed.',
    'longlist': 'List the whole source code for the current function or frame.',
    'retval': 'Print the return value for the last return of a function.',
    'source': 'Try to get source code for the given object and display it.',
    'interact': 'Start an interactive interpreter.',
}

shortcuts = {
    'a': 'args',
    'b': 'break',
    'bt': 'where',
    'cl': 'clear',
    'd': 'down',
    'h': 'help',
    'l': 'list',
    'll': 'longlist',
    'n': 'next',
    'q': 'quit',
    'r': 'return',
    's': 'step',
    'u': 'up',
    'unt': 'until',
    'w': 'where',
}


completion_hints = [
    (('help', ), '<command>'),
    (['l', 'list'], '[<first> [<last>]]'),
    (['cl', 'clear'], '[<filename>:<lineno> | <bpnumber>...]'),
    (('j', 'jump'), '<lineno>'),
    (('restart', 'run'), '[<args>...]'),
    (('p', 'pp', 'whatis'), '<expression>'),
    (('enable', 'disable'), '<bpnumber>...'),
    (('ignore', ), '<bpnumber> <count>'),
    (('condition', ), '<bpnumber> <str_condition>'),
    (('alias', ), '<name> [<command> [<parameter>...]]'),
    (('unalias', ), '<name>'),
    (('b', 'break', 'tbreak'), '([<file>:]<lineno> | <function>) [, <condition>]'),
    (('commands', ), '<bpnumber>'),
    (('up', 'u', 'down', 'd'), '[<count>]'),
    (('until', 'unt'), '[<lineno>]'),
    (('display', 'undisplay'), '[<expression>]'),
    (('source', ), '<expression>'),
    (('commands', ), '[<bpnumber>]'),
]
